using System.Linq;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;

namespace Ruvik
{
    class PHUtils
    {
        static public FreeControllerV3 head;
        static public FreeControllerV3 neck;
        static public FreeControllerV3[] shoulders;
        static public FreeControllerV3[] elbows;
        static public FreeControllerV3[] hands;
        static public FreeControllerV3 chest;
        static public FreeControllerV3 waist;
        static public FreeControllerV3 pelvis;
        static public FreeControllerV3 hip;
        static public FreeControllerV3[] thighs;
        static public FreeControllerV3[] knees;
        static public FreeControllerV3[] feet;
        public static List<FreeControllerV3> activeControllers = new List<FreeControllerV3>();
        static PHController[] phControllers;
        public static Atom activeAtom = null;
        static FreeControllerV3.PositionState[] currentPStates;
        static FreeControllerV3.RotationState[] currentRStates;
        static bool isLocal = true,focusSelected = false;
        public static bool isSymmetryToggled = false;
        public static Transform motionTransform;

        public static float xPosMultiplier = 1.0f;
        public static float yPosMultiplier = 1.0f;
        public static float zPosMultiplier = 1.0f;
        public static float xRotMultiplier = 1.0f;
        public static float yRotMultiplier = 1.0f;
        public static float zRotMultiplier = 1.0f;

        private static void ReverseSelection()
        {
            if (activeControllers.Contains(shoulders[0]))
            {
                phControllers[2].btn.UnPress();
                phControllers[3].btn.Press(false);
            }
            if (activeControllers.Contains(elbows[0]))
            {
                phControllers[4].btn.UnPress();
                phControllers[5].btn.Press(false);
            }
            if (activeControllers.Contains(hands[0]))
            {
                phControllers[6].btn.UnPress();
                phControllers[7].btn.Press(false);
            }
            if (activeControllers.Contains(thighs[0]))
            {
                phControllers[12].btn.UnPress();
                phControllers[13].btn.Press(false);
            }
            if (activeControllers.Contains(knees[0]))
            {
                phControllers[14].btn.UnPress();
                phControllers[15].btn.Press(false);
            }
            if (activeControllers.Contains(feet[0]))
            {
                phControllers[16].btn.UnPress();
                phControllers[17].btn.Press(false);
            }
        }
        public static void ToggleSymmetry()
        {
            isSymmetryToggled = !isSymmetryToggled;
            var btn = PHUI.GetPHButton("Symmetry");
            if (isSymmetryToggled)
            {
                ReverseSelection();
                PHUndo.StoreState();
                btn.GetComponent<Image>().color = new Color(0.8f, 0.2f, 1, 1);
                PHUI.GetPHButton("R.shoulder").interactable = false;
                PHUI.GetPHButton("R.elbow").interactable = false;
                PHUI.GetPHButton("R.hand").interactable = false;
                PHUI.GetPHButton("R.thigh").interactable = false;
                PHUI.GetPHButton("R.knee").interactable = false;
                PHUI.GetPHButton("R.foot").interactable = false;
            }
            else
            {
                btn.GetComponent<Image>().color = Color.white;
                PHUI.GetPHButton("R.shoulder").interactable = true;
                PHUI.GetPHButton("R.elbow").interactable = true;
                PHUI.GetPHButton("R.hand").interactable = true;
                PHUI.GetPHButton("R.thigh").interactable = true;
                PHUI.GetPHButton("R.knee").interactable = true;
                PHUI.GetPHButton("R.foot").interactable = true;
            }
        }
        public static void FocusButtonMethod()
        {
            focusSelected = !focusSelected;
            var btn = PHUI.GetPHButton("Focus");
            if (focusSelected)
                btn.GetComponent<Image>().color = Color.green;
            else
                btn.GetComponent<Image>().color = Color.white;

        }
        public static void SwitchMovementMode()
        {
            isLocal = !isLocal;
            var btn = PHUI.GetPHButton("State");
            if (isLocal)
            {
                btn.GetComponent<Image>().color = Color.white;
                btn.GetComponentInChildren<Text>().text = "local";
                motionTransform = activeAtom.freeControllers[0].transform;
            }
            else
            {
                btn.GetComponent<Image>().color = Color.cyan;
                btn.GetComponentInChildren<Text>().text = "Cam";
                motionTransform = SuperController.singleton.lookCamera.transform;
            }
        }
        public static void PreMoveControllers()
        {
            PHUndo.StoreState();
            currentPStates = new FreeControllerV3.PositionState[activeControllers.Count];
            currentRStates = new FreeControllerV3.RotationState[activeControllers.Count];
            for (int i = 0; i < activeControllers.Count; i++)
            {
                currentPStates[i] = activeControllers[i].currentPositionState;
                currentRStates[i] = activeControllers[i].currentRotationState;
            }
        }
        public static void PostMoveControllers()
        {
            for (int i = 0; i < activeControllers.Count; i++)
            {
                activeControllers[i].currentPositionState = currentPStates[i];
                activeControllers[i].currentRotationState = currentRStates[i];
            }
        }
        public static void InitPHControllers(Atom atom)
        {
            activeControllers.Clear();
            activeAtom = atom;
            motionTransform = activeAtom.freeControllers[0].transform;
            knees = atom.freeControllers.Where(fc => fc.name.Contains("KneeControl")).ToArray();
            hands = atom.freeControllers.Where(fc => fc.name.Contains("HandControl")).ToArray();
            feet = atom.freeControllers.Where(fc => fc.name.Contains("FootControl")).ToArray();
            thighs = atom.freeControllers.Where(fc => fc.name.Contains("ThighControl")).ToArray();
            elbows = atom.freeControllers.Where(fc => fc.name.Contains("ElbowControl")).ToArray();
            shoulders = atom.freeControllers.Where(fc => fc.name.Contains("ArmControl")).ToArray();
            pelvis = atom.freeControllers.First(fc => fc.name.Contains("hip"));
            chest = atom.freeControllers.First(fc => fc.name.Contains("chest"));
            head = atom.freeControllers.First(fc => fc.name.Contains("head"));
            neck = atom.freeControllers.First(fc => fc.name.Contains("neck"));
            hip = atom.freeControllers.First(fc => fc.name == "abdomenControl");
            waist = atom.freeControllers.First(fc => fc.name == "abdomen2Control");

            phControllers = new PHController[]
            {
                new PHController("head",head),
                new PHController("neck",neck),
                new PHController("R.shoulder",shoulders[0]),
                new PHController("L.shoulder", shoulders[1]),
                new PHController("R.elbow", elbows[0]),
                new PHController("L.elbow", elbows[1]),
                new PHController("R.hand", hands[0]),
                new PHController("L.hand",  hands[1]),
                new PHController("chest", chest),
                new PHController("waist", waist),
                new PHController("pelvis", pelvis),
                new PHController("hip", hip),
                new PHController("R.thigh", thighs[0]),
                new PHController("L.thigh", thighs[1]),
                new PHController("R.knee", knees[0]),
                new PHController("L.knee", knees[1]),
                new PHController("R.foot", feet[0]),
                new PHController("L.foot", feet[1]),
                new PHController("name", atom.freeControllers[0]),
            };
        }
        static public void PHButtoOnClickEvent(PHButton phB)
        {
            if (phB.isPressed)
                phB.Press(true);
            else
                phB.UnPress();

        }
        static public UnityAction ResetLink()
        {
            return delegate ()
            {
                ResetLink_method();
            };
        }
        static public UnityAction ResetPose()
        {
            return delegate ()
            {
                ResetPose_method();
            };
        }
        static public void ResetPose_method()
        {
            PHUndo.StoreState();
            ResetLink_method();
            for (int i = 1; i < activeAtom.freeControllers.Length; i++)
            {
                activeAtom.freeControllers[i].Reset();
            }

        }
        static public UnityAction SetControllersState(FreeControllerV3.PositionState ps, FreeControllerV3.RotationState rs)
        {
            return delegate ()
            {
                foreach (var fc in activeControllers)
                {
                    fc.currentPositionState = ps;
                    fc.currentRotationState = rs;
                }
            };
        }
        public static void ResetLink_method()
        {
            foreach (FreeControllerV3 fc in activeAtom.freeControllers)
            {
                fc.SelectLinkToRigidbody(null);
            }
            head.currentPositionState = FreeControllerV3.PositionState.On;
            head.currentRotationState = FreeControllerV3.RotationState.On;

            neck.currentPositionState = FreeControllerV3.PositionState.On;
            neck.currentRotationState = FreeControllerV3.RotationState.On;

            shoulders[0].currentPositionState = FreeControllerV3.PositionState.Off;
            shoulders[1].currentPositionState = FreeControllerV3.PositionState.Off;
            shoulders[0].currentRotationState = FreeControllerV3.RotationState.Off;
            shoulders[1].currentRotationState = FreeControllerV3.RotationState.Off;

            elbows[0].currentPositionState = FreeControllerV3.PositionState.Off;
            elbows[1].currentPositionState = FreeControllerV3.PositionState.Off;
            elbows[0].currentRotationState = FreeControllerV3.RotationState.Off;
            elbows[1].currentRotationState = FreeControllerV3.RotationState.Off;

            hands[0].currentPositionState = FreeControllerV3.PositionState.On;
            hands[1].currentPositionState = FreeControllerV3.PositionState.On;
            hands[0].currentRotationState = FreeControllerV3.RotationState.On;
            hands[1].currentRotationState = FreeControllerV3.RotationState.On;

            chest.currentPositionState = FreeControllerV3.PositionState.On;
            chest.currentRotationState = FreeControllerV3.RotationState.On;

            waist.currentPositionState = FreeControllerV3.PositionState.Off;
            waist.currentRotationState = FreeControllerV3.RotationState.Off;

            pelvis.currentPositionState = FreeControllerV3.PositionState.On;
            pelvis.currentRotationState = FreeControllerV3.RotationState.On;

            hip.currentPositionState = FreeControllerV3.PositionState.Off;
            hip.currentRotationState = FreeControllerV3.RotationState.Off;

            thighs[0].currentPositionState = FreeControllerV3.PositionState.Off;
            thighs[1].currentPositionState = FreeControllerV3.PositionState.Off;
            thighs[0].currentRotationState = FreeControllerV3.RotationState.Off;
            thighs[1].currentRotationState = FreeControllerV3.RotationState.Off;

            knees[0].currentPositionState = FreeControllerV3.PositionState.Off;
            knees[1].currentPositionState = FreeControllerV3.PositionState.Off;
            knees[0].currentRotationState = FreeControllerV3.RotationState.Off;
            knees[1].currentRotationState = FreeControllerV3.RotationState.Off;

            feet[0].currentPositionState = FreeControllerV3.PositionState.On;
            feet[1].currentPositionState = FreeControllerV3.PositionState.On;
            feet[0].currentRotationState = FreeControllerV3.RotationState.On;
            feet[1].currentRotationState = FreeControllerV3.RotationState.On;
        }
        static public UnityAction SetLink()
        {
            return delegate ()
            {
                for (int i = 0; i < activeControllers.Count - 1; i++)
                {
                    Link(activeControllers[i], activeControllers.Last());
                }
            };
        }
        static public UnityAction ClearLink()
        {
            return delegate ()
            {
                foreach (var fc in activeControllers)
                {
                    fc.linkToRB = null;
                    fc.currentPositionState = FreeControllerV3.PositionState.On;
                    fc.currentRotationState = FreeControllerV3.RotationState.On;
                }
            };
        }
        class PHController
        {
            public PHButton btn;
            public PHController(string buttonId, FreeControllerV3 fc)
            {
                btn = new PHButton(PHUI.GetPHButton(buttonId), fc);
            }
        }
        public static void MoveActiveControllers(Vector3 dir, bool isChaingingPosition)
        {
            dir *= Time.deltaTime * PH.sensetivity.val;
            if (isChaingingPosition)
                foreach (var item in activeControllers)
                {
                    item.currentPositionState = FreeControllerV3.PositionState.On;
                    item.MoveControlRelatve(motionTransform.right * (dir.x * xPosMultiplier) + motionTransform.up * (dir.y * yPosMultiplier) + motionTransform.forward * (dir.z * zPosMultiplier));
                }
            else
                foreach (var item in activeControllers)
                {
                    item.currentRotationState = FreeControllerV3.RotationState.On;
                    item.RotateControl(item.transform.right * (dir.x * yRotMultiplier) + item.transform.up * (dir.y * xRotMultiplier) + item.transform.forward * (dir.z * zRotMultiplier), PH.rotationSpeed.val);
                }

        }
        public class PHButton
        {
            public bool isPressed;
            Button btn;
            Image btnImage;
            FreeControllerV3 fc;
            public PHButton(Button btn, FreeControllerV3 fc)
            {
                this.btn = btn;
                this.fc = fc;
                this.btn.onClick.RemoveAllListeners();
                this.btn.onClick.AddListener(delegate () { SetState(); });
                btnImage = btn.GetComponent<Image>();
                btnImage.color = Color.white;
                isPressed = false;
            }
            void SetState()
            {
                if (isPressed)
                    UnPress();
                else
                    Press(true);
            }
            public void Press(bool checkInput)
            {
                if (checkInput && !Input.GetKey(KeyCode.LeftShift))
                    ClearPressedButtons();
                isPressed = true;
                btnImage.color = Color.green;
                if (!activeControllers.Contains(fc))
                {
                    activeControllers.Add(fc);
                }

                if (focusSelected)
                    SuperController.singleton.FocusOnController(activeControllers.Last(),PH.adjustPosition.valNoCallback,PH.alignUpDown.valNoCallback);
            }
            public void UnPress()
            {
                isPressed = false;
                btnImage.color = Color.white;
                if (activeControllers != null)
                    activeControllers.Remove(fc);
            }
        }

        public static void MirrorArmsMethod(bool right)
        {
            int mirror = right ? 0 : 1; int mirrored = right ? 1 : 0;
            shoulders[mirrored].currentPositionState = FreeControllerV3.PositionState.On;
            shoulders[mirrored].currentRotationState = FreeControllerV3.RotationState.On;
            elbows[mirrored].currentPositionState = FreeControllerV3.PositionState.On;
            elbows[mirrored].currentRotationState = FreeControllerV3.RotationState.On;
            hands[mirrored].currentPositionState = FreeControllerV3.PositionState.On;
            hands[mirrored].currentRotationState = FreeControllerV3.RotationState.On;
            shoulders[mirrored].MoveControl(GetInversePosition(shoulders[mirror].transform));
            elbows[mirrored].MoveControl(GetInversePosition(elbows[mirror].transform));
            hands[mirrored].MoveControl(GetInversePosition(hands[mirror].transform));
            shoulders[mirrored].RotateTo(GetInverseRotation(shoulders[mirror].transform));
            elbows[mirrored].RotateTo(GetInverseRotation(elbows[mirror].transform));
            hands[mirrored].RotateTo(GetInverseRotation(hands[mirror].transform));
            shoulders[mirrored].currentPositionState = shoulders[mirror].currentPositionState;
            elbows[mirrored].currentPositionState = elbows[mirror].currentPositionState;
            hands[mirrored].currentPositionState = hands[mirror].currentPositionState;
            shoulders[mirrored].currentRotationState = shoulders[mirror].currentRotationState;
            elbows[mirrored].currentRotationState = elbows[mirror].currentRotationState;
            hands[mirrored].currentRotationState = hands[mirror].currentRotationState;
        }

        public static void MirrorLegsMethod(bool right)
        {
            int mirror = right ? 0 : 1; int mirrored = right ? 1 : 0;
            thighs[mirrored].currentPositionState = FreeControllerV3.PositionState.On;
            thighs[mirrored].currentRotationState = FreeControllerV3.RotationState.On;
            knees[mirrored].currentPositionState = FreeControllerV3.PositionState.On;
            knees[mirrored].currentRotationState = FreeControllerV3.RotationState.On;
            feet[mirrored].currentPositionState = FreeControllerV3.PositionState.On;
            feet[mirrored].currentRotationState = FreeControllerV3.RotationState.On;
            thighs[mirrored].MoveControl(GetInversePosition(thighs[mirror].transform));
            knees[mirrored].MoveControl(GetInversePosition(knees[mirror].transform));
            feet[mirrored].MoveControl(GetInversePosition(feet[mirror].transform));
            thighs[mirrored].RotateTo(GetInverseRotation(thighs[mirror].transform));
            knees[mirrored].RotateTo(GetInverseRotation(knees[mirror].transform));
            feet[mirrored].RotateTo(GetInverseRotation(feet[mirror].transform));
            thighs[mirrored].currentPositionState = thighs[mirror].currentPositionState;
            knees[mirrored].currentPositionState = knees[mirror].currentPositionState;
            feet[mirrored].currentPositionState = feet[mirror].currentPositionState;
            thighs[mirrored].currentRotationState = thighs[mirror].currentRotationState;
            knees[mirrored].currentRotationState = knees[mirror].currentRotationState;
            feet[mirrored].currentRotationState = feet[mirror].currentRotationState;
        }
        public static UnityAction MirrorArmsAction(bool right)
        {
            return delegate ()
            {
                PHUndo.StoreState();
                MirrorArmsMethod(right);
            };
        }
        public static UnityAction MirrorLegsAction(bool right)
        {
            return delegate ()
            {
                PHUndo.StoreState();
                MirrorLegsMethod(right);
            };
        }
        static Vector3 GetInversePosition(Transform trans)
        {
            Vector3 inversePos = Vector3.Reflect(trans.position - activeAtom.freeControllers[0].transform.position, activeAtom.freeControllers[0].transform.right);
            return activeAtom.freeControllers[0].transform.position + inversePos;
        }
        static Quaternion GetInverseRotation(Transform trans)
        {
            return ReflectRotation(trans.rotation, activeAtom.freeControllers[0].transform.right);
        }
        static Quaternion ReflectRotation(Quaternion source, Vector3 normal)
        {
            return Quaternion.LookRotation(Vector3.Reflect(source * Vector3.forward, normal), Vector3.Reflect(source * Vector3.up, normal));
        }
        static public UnityAction LinkAll()
        {
            return delegate ()
            {
                LinkAll_method();
            };
        }
        public static void LinkAll_method()
        {

            for (int i = 0; i < 2; i++)
            {
                Link(shoulders[i], chest);
                Link(elbows[i], shoulders[i]);
                Link(hands[i], elbows[i]);
                Link(thighs[i], hip);
                Link(knees[i], thighs[i]);
                Link(feet[i], knees[i]);
            }
            Link(head, neck);
            Link(neck, chest);
            Link(chest, waist);
            Link(hip, pelvis);
            waist.currentPositionState = FreeControllerV3.PositionState.On;
            waist.currentRotationState = FreeControllerV3.RotationState.On;
        }
        static void Link(FreeControllerV3 a, FreeControllerV3 b)
        {
            a.SelectLinkToRigidbody(b.gameObject.GetComponent<Rigidbody>());
            a.currentPositionState = FreeControllerV3.PositionState.ParentLink;
            a.currentRotationState = FreeControllerV3.RotationState.ParentLink;
        }
        public static void ClearPressedButtons()
        {
            foreach (var controller in phControllers)
            {
                controller.btn.UnPress();
            }
        }

        public static UnityAction SwapArms()
        {
            return delegate ()
            {
                SwapArms_method();
            };
        }
        public static void SwapLegs_method()
        {
            PHUndo.StoreState();
            Vector3 thighPos = Vector3.zero, kneePos = Vector3.zero, footPos = Vector3.zero;
            Quaternion thighRot, kneeRot, footRot;

            FreeControllerV3.PositionState[] prevPStates = new FreeControllerV3.PositionState[3]{
                    thighs[0].currentPositionState,knees[0].currentPositionState,feet[0].currentPositionState
                    };

            FreeControllerV3.RotationState[] prevRStates = new FreeControllerV3.RotationState[3]{
                    thighs[0].currentRotationState,knees[0].currentRotationState,feet[0].currentRotationState
                    };
            thighPos = GetInversePosition(thighs[0].transform);
            kneePos = GetInversePosition(knees[0].transform);
            footPos = GetInversePosition(feet[0].transform);
            thighRot = GetInverseRotation(thighs[0].transform);
            kneeRot = GetInverseRotation(knees[0].transform);
            footRot = GetInverseRotation(feet[0].transform);


            thighs[0].currentPositionState = FreeControllerV3.PositionState.On;
            thighs[0].currentRotationState = FreeControllerV3.RotationState.On;

            knees[0].currentPositionState = FreeControllerV3.PositionState.On;
            knees[0].currentRotationState = FreeControllerV3.RotationState.On;

            feet[0].currentPositionState = FreeControllerV3.PositionState.On;
            feet[0].currentRotationState = FreeControllerV3.RotationState.On;

            thighs[0].MoveControl(GetInversePosition(thighs[1].transform));
            knees[0].MoveControl(GetInversePosition(knees[1].transform));
            feet[0].MoveControl(GetInversePosition(feet[1].transform));
            thighs[0].RotateTo(GetInverseRotation(thighs[1].transform));
            knees[0].RotateTo(GetInverseRotation(knees[1].transform));
            feet[0].RotateTo(GetInverseRotation(feet[1].transform));

            thighs[0].currentPositionState = thighs[1].currentPositionState;
            thighs[0].currentRotationState = thighs[1].currentRotationState;
            feet[0].currentRotationState = feet[1].currentRotationState;
            feet[0].currentPositionState = feet[1].currentPositionState;
            knees[0].currentRotationState = knees[1].currentRotationState;
            knees[0].currentPositionState = knees[1].currentPositionState;

            thighs[1].currentPositionState = FreeControllerV3.PositionState.On;
            thighs[1].currentRotationState = FreeControllerV3.RotationState.On;

            knees[1].currentPositionState = FreeControllerV3.PositionState.On;
            knees[1].currentRotationState = FreeControllerV3.RotationState.On;

            feet[1].currentPositionState = FreeControllerV3.PositionState.On;
            feet[1].currentRotationState = FreeControllerV3.RotationState.On;

            thighs[1].MoveControl(thighPos);
            knees[1].MoveControl(kneePos);
            feet[1].MoveControl(footPos);
            thighs[1].RotateTo(thighRot);
            knees[1].RotateTo(kneeRot);
            feet[1].RotateTo(footRot);

            thighs[1].currentPositionState = prevPStates[0];
            thighs[1].currentRotationState = prevRStates[0];
            knees[1].currentRotationState = prevRStates[1];
            knees[1].currentPositionState = prevPStates[1];
            feet[1].currentRotationState = prevRStates[2];
            feet[1].currentPositionState = prevPStates[2];

        }
        public static void SwapArms_method()
        {
            PHUndo.StoreState();
            Vector3 shoulderPos = Vector3.zero, elbowPos = Vector3.zero, handPos = Vector3.zero;
            Quaternion shoulderRot, elbowRot, handRot;

            FreeControllerV3.PositionState[] prevPStates = new FreeControllerV3.PositionState[3]{
                    shoulders[0].currentPositionState,elbows[0].currentPositionState,hands[0].currentPositionState
                    };

            FreeControllerV3.RotationState[] prevRStates = new FreeControllerV3.RotationState[3]{
                    shoulders[0].currentRotationState,elbows[0].currentRotationState,hands[0].currentRotationState
                    };
            shoulderPos = GetInversePosition(shoulders[0].transform);
            elbowPos = GetInversePosition(elbows[0].transform);
            handPos = GetInversePosition(hands[0].transform);
            shoulderRot = GetInverseRotation(shoulders[0].transform);
            elbowRot = GetInverseRotation(elbows[0].transform);
            handRot = GetInverseRotation(hands[0].transform);


            shoulders[0].currentPositionState = FreeControllerV3.PositionState.On;
            shoulders[0].currentRotationState = FreeControllerV3.RotationState.On;

            elbows[0].currentPositionState = FreeControllerV3.PositionState.On;
            elbows[0].currentRotationState = FreeControllerV3.RotationState.On;

            hands[0].currentPositionState = FreeControllerV3.PositionState.On;
            hands[0].currentRotationState = FreeControllerV3.RotationState.On;

            shoulders[0].MoveControl(GetInversePosition(shoulders[1].transform));
            elbows[0].MoveControl(GetInversePosition(elbows[1].transform));
            hands[0].MoveControl(GetInversePosition(hands[1].transform));
            shoulders[0].RotateTo(GetInverseRotation(shoulders[1].transform));
            elbows[0].RotateTo(GetInverseRotation(elbows[1].transform));
            hands[0].RotateTo(GetInverseRotation(hands[1].transform));

            shoulders[0].currentPositionState = shoulders[1].currentPositionState;
            shoulders[0].currentRotationState = shoulders[1].currentRotationState;
            hands[0].currentRotationState = hands[1].currentRotationState;
            hands[0].currentPositionState = hands[1].currentPositionState;
            elbows[0].currentRotationState = elbows[1].currentRotationState;
            elbows[0].currentPositionState = elbows[1].currentPositionState;

            shoulders[1].currentPositionState = FreeControllerV3.PositionState.On;
            shoulders[1].currentRotationState = FreeControllerV3.RotationState.On;

            elbows[1].currentPositionState = FreeControllerV3.PositionState.On;
            elbows[1].currentRotationState = FreeControllerV3.RotationState.On;

            hands[1].currentPositionState = FreeControllerV3.PositionState.On;
            hands[1].currentRotationState = FreeControllerV3.RotationState.On;

            shoulders[1].MoveControl(shoulderPos);
            elbows[1].MoveControl(elbowPos);
            hands[1].MoveControl(handPos);
            shoulders[1].RotateTo(shoulderRot);
            elbows[1].RotateTo(elbowRot);
            hands[1].RotateTo(handRot);

            shoulders[1].currentPositionState = prevPStates[0];
            shoulders[1].currentRotationState = prevRStates[0];
            elbows[1].currentRotationState = prevRStates[1];
            elbows[1].currentPositionState = prevPStates[1];
            hands[1].currentRotationState = prevRStates[2];
            hands[1].currentPositionState = prevPStates[2];

        }

        public static UnityAction SwapLegs()
        {
            return delegate ()
            {
                SwapLegs_method();
            };
        }
    }
    class MovementButtons : MonoBehaviour, UnityEngine.EventSystems.IPointerEnterHandler, UnityEngine.EventSystems.IPointerExitHandler
    {
        bool isHeld = false;
        bool isMoused = false;
        char buttonAxis;
        bool buttonMotion;
        public void OnPointerEnter(UnityEngine.EventSystems.PointerEventData eventData)
        {
            isMoused = true;
        }
        public void OnPointerExit(UnityEngine.EventSystems.PointerEventData eventData)
        {
            isMoused = false;
        }
        void Start()
        {
            buttonAxis = gameObject.name[4];
            buttonMotion = gameObject.name[3] == 'P' ? true : false;
        }
        public void Update()
        {
            if (PHUtils.activeControllers.Count <= 0) return;

            if (!isHeld && isMoused && Input.GetMouseButtonDown(0))
            {
                isHeld = true;
                PHUtils.PreMoveControllers();
            }

            if (isHeld)
            {
                Vector3 dir = Vector3.zero;
                float mouseMovement = Input.GetAxis("Mouse X");
                switch (buttonAxis)
                {
                    case 'X':
                        dir = Vector3.right * mouseMovement;
                        break;
                    case 'Y':
                        dir = Vector3.up * mouseMovement;
                        break;
                    case 'Z':
                        dir = Vector3.forward * mouseMovement;
                        break;
                }
                PHUtils.MoveActiveControllers(dir, buttonMotion);
            }

            if (isHeld && Input.GetMouseButtonUp(0))
            {
                isHeld = false;
                PHUtils.PostMoveControllers();
            }

        }
    }
}
